# Table

-   category: Components
-   family: DataDisplay
-   chinese: 表格
-   cols: 1
-   type: Display

---

## Guide

Table are responsible for presenting data as highly customizable and accessible HTML table whose core function is to present structured data in a tabular manner. Then you can use various parameters to add some features to the table, such as sorting, filtering, scrolling, lock columns, and so on.

### Basic Usage

The basic Table contains rows and columns, use Table.Column to define the column, use the dataSource property passed to create rows.

The following code will create a two-column data table:

```js
import { Table } from '@alifd/next';

const dataSource = [{id: 1, time: '2016'}];
ReactDOM.render(
    <Table dataSource={dataSource}>
        <Table.Column title="Id" dataIndex="id"/>
        <Table.Column title="Time" dataIndex="time"/>
    </Table>, mountNode);
```

#### Column Configuration

Table.Column provides a lot of configuration properties for custom columns, the most common is the use of `cell` custom cell rendering. Other configurations can refer to the following Table.Column API.

The following code will make cells render different views based on values:

```js
import { Table } from '@alifd/next';

const dataSource = [{id: 1, time: '2016'}];
const renderTime = value => {
    if (value === '2016') {
        return 'this year';
    }
    return value;
};
ReactDOM.render(
    <Table dataSource={dataSource}>
        <Table.Column title="Id" dataIndex="id"/>
        <Table.Column title="Time" dataIndex="time" cell={renderTime}/>
    </Table>, mountNode);
```

#### Multiple Table Header

Use Table.ColumnGroup to wrap Table.Column to create a table with multiple headers:

```js
import { Table } from '@alifd/next';

const dataSource = [{id: 1, time: '2016'}];
ReactDOM.render(
    <Table dataSource={dataSource}>
        <Table.ColumnGroup>
            <Table.Column title="Id" dataIndex="id"/>
            <Table.Column title="Time" dataIndex="time"/>
        </Table.ColumnGroup>
        <Table.ColumnGroup>
            <Table.Column title="Id" dataIndex="id"/>
        </Table.ColumnGroup>
    </Table>, mountNode);
```

### Know Issues

#### Group Table does not support displaying background color in hover state and selected state, unable to merge cells

#### Table can not merge cell if enable locking column

## API

### Table

| Param | Descripiton  | Type  | Default Value |
| ------------------- | ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | --------------- | --------- |
| tableLayout         | table-layout attribute of table element<br><br>**options**:<br>'fixed', 'auto'                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | Enum          | -        |
| tableWidth          | totoal width of table                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | Number        | -        |
| dataSource          | data source shown in the table                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Array           | \[]       |
| onRowClick          | callback triggered when click each row<br><br>**signatures**:<br>Function(record: Object, index: Number, e: Event) => void<br>**params**:<br>_record_: {Object} the data corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>_e_: {Event} event object                                                                                                                                                                                                                                                                                                                                                                                                                               | Function        | () => { } |
| onRowMouseEnter     | callback triggered when mouse enter each row<br><br>**signatures**:<br>Function(record: Object, index: Number, e: Event) => void<br>**params**:<br>_record_: {Object} the data corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>_e_: {Event} event object                                                                                                                                                                                                                                                                                                                                                                                                                           | Function        | () => { } |
| onRowMouseLeave     | callback triggered when mouse leave each row<br><br>**signatures**:<br>Function(record: Object, index: Number, e: Event) => void<br>**params**:<br>_record_: {Object} the data corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>_e_: {Event} event object                                                                                                                                                                                                                                                                                                                                                                                                                            | Function        | () => { } |
| onSort              | callback triggered when sort<br><br>**signatures**:<br>Function(dataIndex: String, order: String) => void<br>**params**:<br>_dataIndex_: {String} the specified sorted field<br>_order_: {String} order, whether is 'desc' or 'asc'                                                                                                                                                                                                                                                                                                                                                                                                                                           | Function        | () => { } |
| onFilter            | callback triggered when click filter confirmation button<br><br>**signatures**:<br>Function(filterParams: Object) => void<br>**params**:<br>_filterParams_: {Object} filter params                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | Function        | () => { } |
| onResizeChange      | callback triggered when resize column<br><br>**signatures**:<br>Function(dataIndex: String, value: Number) => void<br>**params**:<br>_dataIndex_: {String} the specify resize field<br>_value_: {Number} column width variation                                                                                                                                                                                                                                                                                                                                                                                                                                                            | Function        | () => { } |
| rowProps         | get the properties of row, the return value will not work if it is in conflict with other properties<br><br>**signatures**:<br>Function(record: Object, index: Number) => Object<br>**params**:<br>_record_: {Object} the data corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>**returns**:<br>{Object} properties of Row<br>                                                                                                                                                                                                                                                                                                                                                                                                       | Function        | () => { } |
| getCellProps        | get the properties of cell, through which you can merge cells<br><br>**signatures**:<br>Function(rowIndex: Number, colIndex: Number, dataIndex: String, record: Object) => Object<br>**params**:<br>_rowIndex_: {Number} the index corresponding to the row<br>_colIndex_: {Number} the index corresponding to the column<br>_dataIndex_: {String} the field corresponding to the column<br>_record_: {Object} the record corresponding to the row<br>**returns**:<br>{Object} properties of td<br>                                                                                                                                                                                                                                                                                         | Function        | () => { } |
| hasBorder           | whether the table has a border                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Boolean         | true      |
| hasHeader           | whether the table has header                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Boolean         | true      |
| isZebra             | whether the table has zebra style                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Boolean         | false     |
| loading             | whether data is loading                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Boolean         | false     |
| loadingComponent    | customized Loading component <br><br>**signatures**:<br>Function() => void                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | Function        | -         |
| filterParams        | currently filtered keys, use this property to control which menu in the table's header filtering options is selected, in the format {dataIndex: {selectedKeys:\[]}}<br> Example: <br> Suppose you want to control dataIndex as Select <br><Table filterParams={{id: {selectedKeys: ['one']}}}/> for the menu item whose key is one in the filtering menu of the id column.                                                                                                                                                                                                                                                                                                                                                                                                                                | Object          | -         |
| sort                | the currently sorted field, use this property to control the sorting of the table's fields in the format {dataIndex: 'asc'}                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | Object          | -         |
| sortIcons           | customize sortIcons, to set top and bottom layout e.g.`{desc: <Icon style={{top: '6px', left: '4px'}} type={'arrow-down'} size="small" />, asc: <Icon style={{top: '-6px', left: '4px'}} type={'arrow-up'} size="small" />}`                                                                                                                                                                                                                                                                                                                                                                                                                   | Object          | -        |
| columns             | equals to Table.Column, but Table.Column has a higher priority                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         | Array         | -        |
| emptyContent        | content when the table is empty                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | ReactNode       | -         |
| primaryKey          | the primary key of data in the dataSource, if the attribute in the given data source does not contain the primary key, it will cause selecting all                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | String          | 'id'      |
| expandedRowRender   | rendering function for expanded row<br><br>**signatures**:<br>Function(record: Object, index: Number) => Element<br>**params**:<br>_record_: {Object} the record corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>**returns**:<br>{Element} render content<br>                                                                                                                                                                                                                                                                                                                                                                                                                              | Function        | -         |
| rowExpandable       | row can expand or not<br><br>**signatures**:<br>Function(record: Object, index: Number) => Boolean<br>**params**:<br>_record_: {Object} the record corresponding to the row<br>_index_: {Number} the index corresponding to the row<br>**returns**:<br>{Boolean} expandable<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | Function      | -        |
| expandedRowIndent   | indent of expanded row                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Array           | -         |
| openRowKeys         | (under control)keys of expanded row, usually used with onRowOpen                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         | Array           | -         |
| onRowOpen           | callback triggered when expand row<br><br>**signatures**:<br>Function(openRowKeys: Array, currentRowKey: String, expanded: Boolean, currentRecord: Object) => void<br>**params**:<br>_openRowKeys_: {Array} key of expanded row<br>_currentRowKey_: {String} key of current clicked row<br>_expanded_: {Boolean} whether is expanded<br>_currentRecord_: {Object} the data corresponding to the current clicked row                                                                                                                                                                                                                                                                                            | Function        | -         |
| hasExpandedRowCtrl  | whether to show the + button that clicks to expand additional row                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | Boolean         | -         |
| getExpandedColProps | get properties of expanded row<br><br>**signatures**:<br>Function() => void                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | Function        | -         |
| fixedHeader         | whether the table header is fixed, this property is used with maxBodyHeight. When the height of the content area exceeds maxBodyHeight, a scroll bar appears in the content area.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | Boolean         | -         |
| maxBodyHeight       | the height of the largest content area, when `fixedHeader` is `true`, scroll bars will appear above this height                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | Number/String          | -         |
| rowSelection        | whether to enable selection mode <br><br>**properties**:<br>_getProps_: {Function} `Function(record, index)=>Object` get default attributes of selection<br>_onChange_: {Function} `Function(selectedRowKeys:Array, records:Array)` callback triggered when selection change, **Note:** where records only contains the data of the current dataSource, it is likely to be less than the length of the selectedRowKeys.<br>_onSelect_: {Function} `Function(selected:Boolean, record:Object, records:Array)` callback triggered when user select<br>_onSelectAll_: {Function} `Function(selected:Boolean, records:Array)` callback triggered when user select all<br>_selectedRowKeys_: {Array} if this property is set, the rowSelection is controlled, and the received value is the value of the primaryKey of the row's data.<br>_mode_: {String} the mode of selection, the optional value is `single`, `multiple`, the default is `multiple`<br>_columnProps_: {Function} `Function()=>Object` props of checkbox columns which inherit from `Table.Column`<br>_titleProps_: {Function} `Function()=>Object` props of checkbox columns header, works only in `multiple` mode <br>_titleAddons_: {Function} `Function()=>Node` element added to checkbox columns header, works in both `single` and `multiple` mode  | Object          | -         |
| stickyHeader        | whether the table header is sticky                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            | Boolean         | -         |
| offsetTop           | affix triggered after the distance top reaches the specified offset                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Number          | -         |
| affixProps          | properties of Affix                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            | Object          | -         |
| indent              | indented in tree mode, work only when isTree is true                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Number          | -         |
| isTree              | enable the tree mode of Table, the received data format contains children and renders it into a tree table.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Boolean         | -         |
| useVirtual          | whether use virtual scroll                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | Boolean         | -         |
| onBodyScroll        | callback triggered when body scroll<br><br>**signatures**:<br>Function() => void                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | Function        | -         |
| crossline           | show corssline when hover，which is suitable for the scene where the header is complex and needs to be classified                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Boolean       | false    |

### Table.Column

| Param | Description  | Type  | Default Value |
| ---------- | ----------------------------------------------------- | ------------------------------- | ---------------- |
| dataIndex  | specify the column corresponding field, support the fast value of `a.b` format                               | String                          | -                |
| cell       | cell rendering logic<br>Function(value, index, record) => Element   | ReactElement/ReactNode/Function | (value) => value |
| title      | content of table header                                               | ReactElement/ReactNode/Function | -                |
| htmlTitle  | the props of title, which will be writen on header cells' dom                                                                                                                                                                                    | String                          | -                                 |
| sortable   | whether to support sorting                                                | Boolean                         | -                |
| width      | width of column，width needs to be set in lock style                                  | Number/String                   | -                |
| align      | alignment of cell<br><br>**options**:<br>'left', 'center', 'right' | Enum                            | -                |
| alignHeader | alignment of header cell, value of align by default<br><br>**options**:<br>'left', 'center', 'right'                                             | Enum                            | -                |
| filters    | generates a title filter menu in the format `[{label:'xxx', value:'xxx'}]`         | Array&lt;Object>                | -                |
| filterMode | whether the filtering mode is single or multiple selection<br><br>**options**:<br>'single', 'multiple'  | Enum                            | 'multiple'       |
| filterMenuProps | the props passed to Menu in filter mode, extend `Menu`'s API <br><br>**options**:<br>_subMenuSelectable_: {Boolean} default `false` subMenu can be selected or not<br>_isSelectIconRight_: {Boolean} default `false` select icon in right or not. (icon on SubMenu always in left) | Object                          | {     subMenuSelectable: false } |
| lock       | whether the lock column is supported, the options are `left`, `right`, `true`                    | Boolean/String                  | -                |
| resizable  | whether to support column resizing, when this value is set to true, the layout of the table will be modified to fixed              | Boolean                         | false            |
| colSpan    | theader can merge cell by this API, 0 means no th of this column                                                                                                                                                                            | Number                          | -                                 |

### Table.ColumnGroup

| Param | Description  | Type  | Default Value |
| ----- | ------- | ------------------------------- | -------------- |
| title | content of table header | ReactElement/ReactNode/Function | 'column-group' |

### Table.GroupHeader

| Param | Description  | Type  | Default Value |
| -------------------- | --------------------------- | ------------------------------- | -------- |
| cell                 | cell rendering logic                      | ReactElement/ReactNode/Function | () => '' |
| hasChildrenSelection | whether to render selection on Children    | Boolean                         | false    |
| hasSelection         | whether to render selection on GroupHeader | Boolean                         | true     |
| useFirstLevelDataWhenNoChildren | use the first level data when there is no children in dataSouce | Boolean                         | false    |

### Table.GroupFooter

| Param | Description  | Type  | Default Value |
| ---- | ------ | ------------------------------- | -------- |
| cell | cell rendering logic | ReactElement/ReactNode/Function | () => '' |

